/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamgain.h"
#include "gdamdispatch.h"
#include <gdam/gdam-protocol.h>


static int         gdam_gain_read               (GdamSource*	  gain,
                                       	         GdamSample*	  buf,
				       	         int		  n);

static void        gdam_gain_class_init         (GdamFilterClass* klass);
static void        gdam_gain_init               (GdamGain*      gain);

static GtkObjectClass* parent_class;

GtkType gdam_gain_get_type()
{
	static GtkType gain_type = 0;
	if (!gain_type) {
		static const GtkTypeInfo gain_info = {
			"GdamGain",
			sizeof(GdamGain),
			sizeof(GdamFilterClass),
			(GtkClassInitFunc) gdam_gain_class_init,
			(GtkObjectInitFunc) gdam_gain_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		parent_class = gtk_type_class(GDAM_TYPE_FILTER);
		gain_type = gtk_type_unique(GDAM_TYPE_FILTER, &gain_info);
	}
	return gain_type;
}
static void gdam_gain_arg_set_gain(GtkObject* object, GdamArg* arg)
{
	gdouble gain = GDAM_ARG_DOUBLE(arg);
	gdam_gain_set_level(GDAM_GAIN(object),
	                    (GdamFixed)(gain * GdamFixed_ONE));
}
static void gdam_gain_arg_get_gain(GtkObject* object, GdamArg* arg)
{
	gdouble gain = GDAM_GAIN(object)->regainer_param;
	gain /= GdamFixed_ONE;
	GDAM_ARG_DOUBLE(arg) = gain;
}

static void gdam_gain_class_init(GdamFilterClass* klass)
{
	GdamSourceClass* source_class = (GdamSourceClass*)klass;
	source_class->read = gdam_gain_read;

	gdam_arg_register(GTK_OBJECT_CLASS(klass),
	                  "gain",
			  GDAM_ARG_TYPE_DOUBLE,
			  gdam_gain_arg_set_gain,
			  gdam_gain_arg_get_gain,
			  GDAM_ARG_VITAL);
}

static void gdam_sample_regain_null(GdamSample* buf,
                                    int buf_len,
                                    GdamFixed unused)
{
}

static void gdam_gain_init(GdamGain* gain)
{
	gain->regainer = gdam_sample_regain_null;
	gain->regainer_param = GdamFixed_ONE;
}

static int gdam_gain_read(GdamSource*	source, GdamSample* buf, int n)
{
	GdamGain* gain = GDAM_GAIN(source);
	GdamFilter* filter = GDAM_FILTER(source);
	GdamSource* filtered;
	int num_read;
	g_return_val_if_fail(gain != NULL, 0);
	filtered = filter->filtered;
	if (!filtered)
		return 0;
	num_read = gdam_source_read(filtered, buf, n);
	if (num_read > 0)
		(*gain->regainer)(buf, num_read, gain->regainer_param);
	return num_read;
}

void gdam_gain_set_level(GdamGain* gain, GdamFixed fixed)
{
	if (fixed == GdamFixed_ONE)
		gain->regainer = gdam_sample_regain_null;
	else
		gain->regainer = gdam_sample_scale;
	gain->regainer_param = fixed;
}

